<?php

/**
 * Класс для интеграции с prodalet.ru
 */
class ProdaletTracker
{
    public $version = '3';

    public $debug = false;


    public $first_name = null;
    public $last_name = null;
    public $contact_email = null;
    public $contact_phone = null;
    public $plt_visitor_id = null;

    public $charset = 'utf-8';
    public $timeout = 1;
    public $connect_timeout = 1;
    public $error = false;
    private $api_key;
    private $api_secret;
    public $guest_uid = false; //переменная ProdaLet в которую записывается ID клиента в CMS, если посетитель авторизован
    public $visitor_uid = false;
    private $visitor_info = false;
    private $referrer = false;
    private $url = false;
    private $domain = false;
    private $generated_uid = false;
    private $from_prodalet = '';

    private static $hosts = [
        'prod' => 'https://prodalet.ru',
        'dev' => 'https://dev.prodalet.ru',
        'local' => 'http://prodalet.int',
    ];

    const COOKIE_EXPIRES = 2160000; //Время жизни наших кук
    const COOKIE_NAME_HAS_CART = 'plt_has_cart'; //Кука, что у посетителя есть корзина
    const NEED_CHECK = 'plt_need_check'; //Кука, что продалету надо проверить, показывать виджет или нет
    const COOKIE_CART_DATA = 'plt_cart_data'; //
    const COOKIE_NAME_HAS_UNPAID_ORDER = 'plt_has_unpaid_order'; //


    public static function isDbg()
    {

        if (!defined('PLT_DBG')) {
            $dbg = false;
            $host = waRequest::server('HTTP_HOST');


            if (preg_match('~\.int$~', $host)) {
                $dbg = true;
            }

            define('PLT_DBG', $dbg);
        }

        return PLT_DBG;

    }

    public static function getProdaletUrl()
    {
        $host = waRequest::server('HTTP_HOST');

        if (self::isDbg()) {
            return self::$hosts['local'];
        } elseif (preg_match('~\.dev\.prodalet\.ru$~', $host)) {
            return self::$hosts['dev'];
        } else {
            return self::$hosts['prod'];
        }
    }


    /**
     *
     * @param type $api_key
     * @param type $domain
     * @param type $guest_uid
     * @param type $visitor_uid
     * @param type $visitor_info структура с параметрами клиента:
     * {
     * first_name: 'Имя',
     * last_name: 'Фамилия',
     * email: 'email',
     * phone: 'телефон',
     * date_of_birth: 'Y-m-d',
     * gender: 'male',
     * language: 'ru',
     * custom_field_1: 'Своя переменная 1',
     * custom_field_2: 'Своя переменная 2',
     * ...
     * }
     * @param type $referrer
     * @param type $url
     */
    public function __construct($api_key, $domain = false, $guest_uid = false, $visitor_uid = false, $visitor_info = false, $referrer = false, $url = false)
    {
        $this->api_key = (string)$api_key['api_key'];
        $this->api_secret = (string)$api_key['api_secret'];

        // $domain = ($domain == null) ? $_SERVER["HTTP_HOST"] : $domain; - для всех в целом
        $domain = ($domain == false) ? waRequest::server('HTTP_HOST') : $domain;

        $domain_encoding = mb_detect_encoding($domain, array('UTF-8', 'windows-1251'));
        $this->domain = (string)mb_strtolower((($domain_encoding == 'UTF-8') ? $domain : iconv($domain_encoding, 'UTF-8', $domain)), 'UTF-8');

        $this->guest_uid = (string)$guest_uid;
        $this->visitor_info = $visitor_info;
        $this->visitor_uid = (string)$visitor_uid;
        $this->referrer = (string)$referrer;
        $this->url = (string)$url;

        if (!$this->guest_uid and !$this->visitor_uid) {
            $this->guest_uid = uniqid();
            $this->generated_uid = true;
        }


        try {
            if (!empty($this->visitor_uid)) {
                $contact = new waContact($this->visitor_uid);


                $fields = array(
                    'first_name' => $contact->get('firstname'),
                    'last_name' => $contact->get('lastname'),
                    'phone' => (($phone = $contact->get('phone') and isset($phone[0])) ? $phone[0]['value'] : false),
                    'email' => (($email = $contact->get('email') and isset($email[0])) ? $email[0]['value'] : false),

                );

                $this->visitor_info = array();
                foreach ($fields as $key => $value) {
                    if ($value) $this->visitor_info[$key] = $value;
                }


            }

        } catch (Exception $e) {

        }

        //Если в куках указан ID визитора, уже определенный ранее - то устанавливаем его
        $cookie_PLT_VID = waRequest::cookie('PLT_VID');

        if (!empty($cookie_PLT_VID)) {
            $this->visitor_info['prodalet_visitor_id'] = waRequest::cookie('PLT_VID');
        }


    }


    /*Базовый запрос - вызывается во всех событиях, перед выполнением кода этого события
    например в начале eventProductView(), eventAddToCart() и так далее
    дополняем POST данные для события необходимыми данными для нашего API
    */
    private function formBasicRequest()
    {
        $post = array(); //Создаем пустой массив для заполнения
        $post["api_key"] = $this->api_key;
        $post["api_secret"] = $this->api_secret;

        if ($this->guest_uid) {
            $post["guest_uid"] = $this->guest_uid;
        } else {
            $post["guest_uid"] = false;
        }
        if ($this->visitor_uid) {
            $post["visitor_uid"] = $this->visitor_uid;
        } else {
            $post['visitor_uid'] = false;
        }

        if ($this->contact_email) {
            $post["contact_email"] = $this->contact_email;
        } else {
            $post['contact_email'] = false;
        }
        if ($this->contact_phone) {
            $post["contact_phone"] = $this->contact_phone;
        } else {
            $post['contact_phone'] = false;
        }
        if ($this->first_name) {
            $post["first_name"] = $this->first_name;
        } else {
            $post['first_name'] = false;
        }
        if ($this->last_name) {
            $post["last_name"] = $this->last_name;
        } else {
            $post['last_name'] = false;
        }


        $post["domain"] = $this->domain; //Текущий домен сайта
        $post['api_type'] = 'shopscript'; //Тип cms откуда идет обращение к апи


        //Передаем источник перехода посетителя
        if ($this->referrer) {
            $post["referrer"] = $this->referrer;
        }
        //Текущий урл, откуда вызов
        if ($this->url) {
            $post["url"] = "http://" . $this->url;
            $post["host"] = $this->url;
        }


        //Массив показанных виджетов и когда они были показаны.
        $plt_cookie_Z = waRequest::cookie();
        if (!empty($plt_cookie_Z)) {
            foreach ($plt_cookie_Z as $k => $v) {
                if (preg_match('~^plt_shown_recently____([a-z0-9]+)$~u', $k, $m)) {
                    $post['widgets_shown_recently'][$m[1]] = $v;

                }
            }
        }

        if (is_array($this->visitor_info) && count($this->visitor_info) > 0)
        {
            $post["visitor_info"] = $this->visitor_info;
        }

        return $post;
    }

    /**
     * Обработчик показа карточки товара в магазине
     * @param string $product_id внутренний ID товара в магазине
     * @param string $product_name название товара
     * @param string $product_url абсолютный URL к странице товара
     * @param string $product_image абсолютный URL к изображению товара
     */
    public function eventProductView($product_id, $product_name = false, $product_url = false, $product_image = false)
    {
        //Формируем базовые поля, с дополненными данными
        $post = $this->formBasicRequest();

        //Заполняем маасив post - который потом передастся к нам сервер
        $post["type"] = "view_product"; //Тип события для нашего API
        $post["properties"]["product_id"] = (string)$product_id; //ID товара в магазине
        if ($product_name) $post["properties"]["product_name"] = (string)$product_name; //Название товара  в магазине
        if ($product_url) $post["properties"]["product_url"] = (string)$product_url; //URL товара  в магазине, абослютный путь
        if ($product_image) $post["properties"]["product_image"] = (string)$product_image; //URL изображения товара  в магазине

        return $this->sendEvent($post);
    }


    public function getCookieArray($name, $guest_id, $visitor_id)
    {
        try {
            if (waRequest::cookie($name)) {
                $plt_track = unserialize(waRequest::cookie($name));
            } else {
                $plt_track = array();
            }

        } catch (\Exception $e) {
            $plt_track = array();
        }

        if (!empty($plt_track['uid'])) {
            if (!$visitor_id || ($visitor_id != $plt_track['uid'])) {
                $plt_track = array();
            }
        } elseif (($guest_id != $plt_track['gid'])) {
            $plt_track = array();
        }

        $plt_track['uid'] = $visitor_id;
        $plt_track['gid'] = $guest_id;

        if (!isset($plt_track['data'])) {
            $plt_track['data'] = array();
        }

        return $plt_track;
    }


    public function setCookie($name, $plt_track, $serialize = false)
    {

        if ($serialize) $plt_track = serialize($plt_track);

        if (!is_scalar($plt_track)) return false;

        setcookie($name, $plt_track, time() + self::COOKIE_EXPIRES, '/', waRequest::server('HTTP_HOST'));

    }


    /**
     *
     * @param type $product_id - внутренний ID товара
     * @param type $qnt количество
     * @param type $price цена
     * @param type $product_name название
     * @param type $product_url абсолютный URL товара
     * @return boolean
     */
    public function eventAddToCart($product_id, $qnt, $price, $product_name = false, $product_url = false)
    {
        $post = $this->formBasicRequest();
        $post["type"] = "add_to_cart";
        $post["properties"]["product_id"] = (string)$product_id;
        $post["properties"]["qnt"] = $qnt;
        $post["properties"]["price"] = $price;
        if ($product_name) $post["properties"]["product_name"] = (string)$product_name;
        if ($product_url) $post["properties"]["product_url"] = (string)$product_url;
        $post['plt_user_data'] = (string)waRequest::cookie('plt_user_data');

        //Устанавливаем, что у посетителя есть корзина
        $this->setCookie(self::COOKIE_NAME_HAS_CART, 1);

        return $this->sendEvent($post);
    }


    public function eventUpdateCart($order_list)
    {

        if (defined('PLT_UPDATE_CART_FROM_LETTER')) return true;

        $post = $this->formBasicRequest();
        //Тип события
        $post["type"] = "update_cart";
        $post['plt_user_data'] = (string)waRequest::cookie('plt_user_data');
        $properties = array();
        //Товары
        $properties["items"] = $order_list;

        //Данные по корзине
        $post["properties"] = $properties;

        //Считаем сумму денег в корзине
        $summ = 0.0;
        if (!empty($order_list) && is_array($order_list)) {
            foreach ($order_list as $item) {
                if (!empty($item['price'])) {
                    $qnt = 1;
                    if (!empty($item['qnt'])) $qnt = (int)$item['qnt'];
                    $summ += (float)$item['price'] * $qnt;
                }
            }
        }


        //Устанавливаем куки, что у нас есть корзина суммой ХХХ
        $this->setCookie(self::COOKIE_NAME_HAS_CART, $summ);

        //Отправляем данные на ProdaLet по апи
        return $this->sendEvent($post);
    }


    /*
     * Обновление информации о заказе
     */
    public function eventOrderUpdate($order_id, $state, $revenue = false, $order_list = false)
    {


        $post = $this->formBasicRequest();
        $post["type"] = "order_update";
        $properties = array();
        $properties["order_id"] = (string)$order_id;
        $properties["status"] = (string)$state;

        if ($revenue !== false) $properties["revenue"] = $revenue;

        if (is_array($order_list)) $properties["items"] = $order_list;

        $post["properties"] = $properties;
        unset($post["url"]);
        unset($post["host"]);
        unset($post["path"]);

        return $this->sendEvent($post);
    }


    /*
     * Создание заказа
     * $order_create_date дата создания заказа
     * $exportOldOrders экспортируем заказы из прошлого, при инсталле плагина
     *
     */
    public function eventOrder($order_id, $revenue = false, $order_list = false, $status = false, $order_create_date = false, $exportOldOrders = false)
    {
        $post = $this->formBasicRequest();
        $post["type"] = "purchase";

        $properties = array();
        $properties["order_id"] = (string)$order_id;

        if(!$exportOldOrders) {
            /*Если это не эспорт старых заказов, то надо посмотреть, нами ли мотивирован заказ или нет
            Проверяем, при взаимодействие с нашим виджетом был заказ или нет,
            Наша система в куки устанавливает переменные, если


                - если из письма был переход или другого модуля lead_from_prodalet
                - если было взаимодействие с виджетов нашим в куки ставится lead_from_prodalet_widget

            */
            $properties['from_prodalet'] = (string)waRequest::cookie('lead_from_prodalet');
            $properties['from_prodalet_widget'] = (string)waRequest::cookie('lead_from_prodalet_widget');

            if (defined('FROM_PRODALET') && FROM_PRODALET) {
                $properties['from_prodalet'] = FROM_PRODALET;
            }
        }

        if ($revenue !== false) {
            $properties["revenue"] = $revenue;
        }

        if (is_array($order_list)) {
            $properties["items"] = $order_list;
        }

        if ($status !== false) {
            $properties["status"] = $status;
        }

        if ($order_create_date !== false) {
            $properties["order_create_date"] = $order_create_date;
        }

        $post["properties"] = $properties;
        unset($post["url"]);
        unset($post["host"]);
        unset($post["path"]);

        if(!$exportOldOrders) {
            if ($status == 'new' || $status == 'processing') {
                $this->setCookie(self::COOKIE_NAME_HAS_CART, 0);
                //$this->setCookie(self::COOKIE_CART_DATA, 'no');
                $this->setCookie(self::COOKIE_NAME_HAS_UNPAID_ORDER, 1);
            }

            if ($status == 'paid') {
                $this->setCookie(self::COOKIE_NAME_HAS_UNPAID_ORDER, 0);
            }
        }

        return $this->sendEvent($post);
    }


    /*
     * Для передачи  custom данных
     */
    public function eventCustom($key, $properties = array())
    {
        $post = $this->formBasicRequest();
        $post["type"] = "custom";
        $properties["key"] = (string)$key;
        $post["properties"] = $properties;
        return $this->sendEvent($post);
    }

    /**
     * На данный момент не используется
     * @return boolean
     */
    public function eventUpdateInfo()
    {
        $post = $this->formBasicRequest();
        $post["type"] = "update_info";
        return $this->sendEvent($post);
    }

    /*
     * Передача URL страницы корзины в магазине
     * Вызывается при сохранение настроек плагина
     */
    public function eventSetCartUrl($api_key, $api_secret, $cartUrl)
    {
        $post = $this->formBasicRequest();
        $post["type"] = "setCartUrl";
        $post["pid"] = (string)$api_key;
        $post["cart_url"] = $post['domain'].$cartUrl;
        $post['api_secret'] = $api_secret;
        $post['api_key'] = $api_key;

        return $this->sendEvent($post);

    }

    /*
     * Вызываем ProdaLet - показывая, что клиент установил наш плагин для этого домена
     */
    public function eventPingCms($api_key, $api_secret)
    {
        $post = $this->formBasicRequest();
        $post["type"] = "pingCms";
        $post["pid"] = (string)$api_key;
        $post["cms"] = 'shopscript7';
        $post['project_type'] = 'ecomerce';

        $post['api_secret'] = $api_secret;
        $post['api_key'] = $api_key;

        return $this->sendEvent($post);
    }


    private function getHost()
    {
        return self::getProdaletUrl();
    }

    private function getUrl()
    {
        return $this->getHost() . "/tracker.php";
    }


    /*
     * Функция отправки событий к ProdaLet.ru
     */
    private function sendEvent($post)
    {

        $cookie_val = time();
        if (!empty($post['type']) && strstr($post['type'], 'cart')) {
            $cookie_val = 'cart_' . time();
            /*if(!empty($post['properties'])) {
                setcookie(self::COOKIE_CART_DATA, $this->json_encode($post['properties']), time() + 2, '/', waRequest::server('HTTP_HOST'));
            }*/
        }

        //на 2 секунды устанавливаем куку, что что то произошло с корзиной и надо проверить условия показа виджета, относительно корзины
        setcookie(self::NEED_CHECK, $cookie_val, time() + 2, '/', waRequest::server('HTTP_HOST'));

        $headers = array(
            "Content-Type: application/x-www-form-urlencoded; charset=utf-8"
        );

        //Форматируем массив данных для передачи
        $post = $this->post_encode($post);


        //Отправляем данные к в API ProdaLet
        return $this->send($this->getUrl(), $post, $headers);
    }


    private function post_encode($post)
    {
        $ret = array('api_key' => $post['api_key'], 'api_secret' => $post['api_secret']);

        //Убираем ключи API из массива для json_encode
        if (isset($post['api_key'])) unset($post['api_key']);
        if (isset($post['api_secret'])) unset($post['api_secret']);

        if (!empty($post['visitor_uid'])) $ret['visitor_uid'] = $post['visitor_uid'];
        if (!empty($post['guest_uid'])) $ret['guest_uid'] = $post['guest_uid'];
        $ret['data'] = $this->json_encode($post);
        return $this->build_http_query($ret);
    }

    private function json_encode($text)
    {
        if ($this->charset == "windows-1251") {
            return json_encode($this->json_fix($text));
        } else {
            return json_encode($text);
        }
    }

    private function json_fix($data)
    {
        # Process arrays
        if (is_array($data)) {
            $new = array();
            foreach ($data as $k => $v) {
                $new[$this->json_fix($k)] = $this->json_fix($v);
            }
            $data = $new;
        } # Process objects
        else if (is_object($data)) {
            $datas = get_object_vars($data);
            foreach ($datas as $m => $v) {
                $data->$m = $this->json_fix($v);
            }
        } # Process strings
        else if (is_string($data)) {
            $data = iconv('cp1251', 'utf-8', $data);
        }
        return $data;
    }

    /*
     * Функция отправки данных через CURL
    */
    private function send($url, $post = false, $custom_headers = array(), $method = 'POST', $customrequest = false)
    {
        //Отключение трекинга для debug, путем установки cookie переменной
        $req_cookie_prodalet_track_disable = waRequest::cookie('prodalet_track_disable');
        if (!empty($req_cookie_prodalet_track_disable)) {
            return 'Prodalet tracking disabled';
        }

        $headers = array("Accept:application/json, text/javascript, */*; q=0.01");
        $headers = array_unique(array_merge($headers, $custom_headers));

        $curl = curl_init($url);

        curl_setopt($curl, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $this->connect_timeout);
        curl_setopt($curl, CURLOPT_FAILONERROR, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);

        if ($customrequest) curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($post) {
            if ($method == 'POST') curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $post);

            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        } else {
            curl_setopt($curl, CURLOPT_POST, false);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);

        curl_exec($curl);

        $this->error = curl_error($curl);

        curl_close($curl);

        $this->put_log($url, $method, $post, $custom_headers, $this->error);

        if ($this->error) return $this->error;
        else return true;
    }

    private function build_http_query($query)
    {
        return http_build_query($query);
    }

    private function put_log($url, $method, $post, $headers, $response = true)
    {
        if (!self::isDbg()) {
            return true;
        }

        ob_start();
        if (count($headers) > 0) {
            echo "HEADER: ";
            print_r($headers);
        }
        $str_headers = ob_get_clean();

        ob_start();
        echo "REQUEST DATA: ";
        print_r($post);
        $str_post = ob_get_clean();

        ob_start();
        if ($response !== true) {
            echo "\nRESPONSE: ";
            print_r($response);
        }
        $str_response = ob_get_clean();

        $date = date('Y.m.d H:i:s');
        $row = "{$date}\n{$method} {$url}\n{$str_headers}{$str_post}{$str_response}\n";

        if(class_exists('waLog')) {
            waLog::log($row, 'prodalet_tracker_debug.log');
            ///waLog::dump('Какой-то объект/массив', 'название.log');
        }

        return true;
    }

    //При обращение от ProdaLet.ru к магазину - проверяем валидность данных
    public function checkApiRequest($request, $keys)
    {
        if (!$keys) return false;

        //Ищем данные в запросе, если нет каких то необходимых, то проверка не проходит
        if (empty($request['data'])) return false; //данные в запросе
        if (empty($request['key'])) return false; //ключ API
        if (empty($request['token'])) return false;  //подпись запроса

        //Проверяем корректность
        if ($keys['api_key'] != $request['key']) {
            return false;
        }

        //Проверяем подпись запроса
        $correct_token = md5(json_encode($request['data']) . $keys['api_secret']);
        if ($correct_token != $request['token']) {
            return false;
        }

        return true;
    }

}